; Small test with VBL, rasters and lower border together with blitter sprites that overflow 1 VBL
;
; ae@dhs.nu
; 2012-05-13

numsprites:	equ	17				;Number of blitter sprites to put (64x64 4 planes squares, 11 blocks per vbl) max 17 for now!
show_cpu:	equ	1				;Red colour after blitter pass

		output	.tos

		section	text

begin:
		clr.l	-(sp)				;Enter supervisor mode
		move.w	#32,-(sp)			;
		trap	#1				;
		addq.l	#6,sp				;
		move.l	d0,save_stack			;

		bsr	init
		bsr	set_palette
		bsr	copy_gfx_to_screens
		bsr	init_blitter


		;---------------------------------------
mainloop:
		move.w	vbl_count,d0
.next_vbl:	cmp.w	vbl_count,d0
		beq.s	.next_vbl

		clr.w	vbl_count

		bsr	blit_sprites

		move.l	scr1,d0
		move.l	scr2,scr1
		move.l	d0,scr2

		cmp.b	#$39,$fffffc02.w
		bne.s	mainloop
		;---------------------------------------


		bsr	deinit

		move.l	save_stack,-(sp)		;Exit supervisor
		move.w	#32,-(sp)			;
		trap	#1				;
		addq.l	#6,sp				;

		clr.w	-(sp)				;pterm()
		trap	#1				;



vbl:		move.b	$ffff8a3c.w,.blitter		;save and kill blitter
		bclr	#7,$ffff8a3c.w

		move.w	sr,-(sp)			;save sr and kill interrupts
		move.w	#$2700,sr

		movem.l	d0-a6,-(sp)

		move.l  scr1,d0				;set screen
		lsr.w	#8,d0
		move.l	d0,$ffff8200.w

		bsr	set_palette

		move.l	#rasters,raster_addr		;setup address for rasterlist
		move.w	#198,tb_counter			;number of interrupts before doing lower border

		;Start up Timer B each VBL
		move.w	#$2700,sr			;Stop all interrupts
		clr.b	$fffffa1b.w			;Timer B control (stop)
		bset	#0,$fffffa07.w			;Interrupt enable A (Timer B)
		bset	#0,$fffffa13.w			;Interrupt mask A (Timer B)
		move.b	#1,$fffffa21.w			;Timer B data (number of scanlines to next interrupt)
		bclr	#3,$fffffa17.w			;Automatic end of interrupt
		move.b	#8,$fffffa1b.w			;Timer B control (event mode (HBL))
		move.w	#$2300,sr			;Interrupts back on

		movem.l	(sp)+,d0-a6
		move.w	(sp)+,sr

		addq.w	#1,vbl_count
		move.b	.blitter,$ffff8a3c.w		;restore blitter
		rte
.blitter:	dc.w	0




timer_b:	move.l	a0,-(sp)

		move.l	raster_addr,a0			;get raster colour address
		move.w	(a0)+,$ffff8240.w		;raster colour

		subq.w	#1,tb_counter			;counter to see if it's time for lower border
		beq.s	.lower_border

		addq.l	#2,raster_addr			;next raster colour
		move.l	(sp)+,a0
		rte

		;ok it's time for lower border
.lower_border:
		move.b	$ffff8a3c.w,.blitter		;save and kill blitter
		bclr	#7,$ffff8a3c.w

		clr.b	$fffffa1b.w			;Timer B control (stop)
		move.w	sr,-(sp)
		move.w	#$2100,sr			;enable HBL
		stop	#$2100				;wait for HBL
		move.w	#$2700,sr			;kill interrupts

		move.w	(a0)+,$ffff8240.w		;raster colour

		dcb.w	88,$4e71			;zzz wait for border zapping

		move.b	#0,$ffff820a.w			;60 Hz
		move.w	(a0)+,$ffff8240.w		;raster colour
		move.b	#2,$ffff820a.w			;50 Hz

		move.b	#8,$fffffa1b.w			;Timer B control (event mode (HBL))

		move.w	(sp)+,sr
		addq.l	#6,raster_addr			;three raster colours add
		move.l	(sp)+,a0
		move.b	.blitter,$ffff8a3c.w		;restore blitter
		rte
.blitter:	dc.w	0


blit_sprites:
		move.l	scr1,a0				;base address
		lea	.sprofs,a1			;address offsets

		move.w	#numsprites-1,d7
.loop:
		move.l	a0,a2
		add.l	(a1)+,a2

		move.l	#sprite+32,$ffff8a24.w		;Src
		move.l	a2,$ffff8a32.w			;Dst
		move.w	#64,$ffff8a38.w			;Lines
		move.b	#%10000000,$ffff8a3c.w		;BltCnt

.blitwait:	bset	#7,$ffff8a3c.w			;restart blitter
		nop
		bne.s	.blitwait

		dbra	d7,.loop


		ifne	show_cpu
		move.w	#$0f00,$ffff8240.w
		endc

		rts
.sprofs:
q:		set	0
		rept	17
		dc.l	(160*4*q)+(8*q)
q:		set	q+1
		endr


init_blitter:
		clr.b	$ffff8a3d.w			;Shift
		move.w	#2,$ffff8a20.w			;SX-inc
		move.w	#2,$ffff8a22.w			;SY-inc
		move.w	#2,$ffff8a2e.w			;DX-inc
		move.w	#2+160-32,$ffff8a30.w		;DY-inc
		move.w	#-1,$ffff8a28.w			;EM1
		move.w	#-1,$ffff8a2a.w			;EM2
		move.w	#-1,$ffff8a2c.w			;EM3
		move.b	#%00000010,$ffff8a3a.w		;HTOP
		move.b	#%00000011,$ffff8a3b.w		;LOP
		move.w	#16,$ffff8a36.w			;X-loop
		rts

set_palette:	movem.l	gfx,d0-d7
		movem.l	d0-d7,$ffff8240.w
		rts

copy_gfx_to_screens:
		move.l	scr1,a0
		move.l	scr2,a1
		lea	gfx+32,a2

		move.w	#160*245/4-1,d7
.copy:		move.l	(a2)+,d0
		move.l	d0,(a0)+
		move.l	d0,(a1)+
		dbra	d7,.copy
		rts

init:		move.b	$ffff820a.w,save_refresh	;Save and set refreshrate
		or.b	#%00000010,$ffff820a.w		;

		move.b	$ffff8260.w,save_res		;Save and set resolution
		clr.b	$ffff8260.w			;

		movem.l	$ffff8240.w,d0-d7		;Save palette
		movem.l	d0-d7,save_pal			;

		lea	save_screenadr,a0		;Save screenaddress
		move.b	$ffff8201.w,(a0)+		;
		move.b	$ffff8203.w,(a0)+		;
		move.b	$ffff820d.w,(a0)+		;

		move.l	#screen1+256,d0			;Setup screen addresses
		clr.b	d0				;
		move.l	d0,scr1				;

		move.l	#screen2+256,d0			;
		clr.b	d0				;
		move.l	d0,scr2				;

		move.l  scr1,d0				;Set screenaddress
		lsr.w	#8,d0				;
		move.l	d0,$ffff8200.w			;

		move.b	#$12,$fffffc02.w		;Kill mouse

		move.w	sr,-(sp)
		move.w	#$2700,sr

		move.l	$68.w,save_hbl			;HBL
		move.l	$70.w,save_vbl			;VBL
		move.l	$134.w,save_timer_a		;Timer-A
		move.l	$120.w,save_timer_b		;Timer-B
		move.l	$114.w,save_timer_c		;Timer-C
		move.l	$110.w,save_timer_d		;Timer-D
		move.l	$118.w,save_acia		;ACIA

		lea	save_mfp,a0			;Save vectors and mfp
		move.b	$fffffa01.w,(a0)+		;// datareg
		move.b	$fffffa03.w,(a0)+		;Active edge
		move.b	$fffffa05.w,(a0)+		;Data direction
		move.b	$fffffa07.w,(a0)+		;Interrupt enable A
		move.b	$fffffa13.w,(a0)+		;Interupt Mask A
		move.b	$fffffa09.w,(a0)+		;Interrupt enable B
		move.b	$fffffa15.w,(a0)+		;Interrupt mask B
		move.b	$fffffa17.w,(a0)+		;Automatic/software end of interupt
		move.b	$fffffa19.w,(a0)+		;Timer A control
		move.b	$fffffa1b.w,(a0)+		;Timer B control
		move.b	$fffffa1d.w,(a0)+		;Timer C & D control
		move.b	$fffffa27.w,(a0)+		;Sync character
		move.b	$fffffa29.w,(a0)+		;USART control
		move.b	$fffffa2b.w,(a0)+		;Receiver status
		move.b	$fffffa2d.w,(a0)+		;Transmitter status
		move.b	$fffffa2f.w,(a0)+		;USART data

		move.l	#vbl,$70.w 			;Set VBL
		move.l	#timer_a,$134.w			;Set Timer-A
		move.l	#timer_b,$120.w			;Set Timer-B
		move.l	#timer_c,$114.w			;Set Timer-C
		move.l	#timer_d,$110.w			;Set Timer-D
		move.l	#acia,$118.w			;Set ACIA
		move.l	#hbl,$68.w 			;Set HBL

		clr.b	$fffffa07.w			;Interrupt enable A (Timer-A & B)
		clr.b	$fffffa13.w			;Interrupt mask A (Timer-A & B)
		clr.b	$fffffa09.w			;Interrupt enable B (Timer-C & D)
		clr.b	$fffffa15.w			;Interrupt mask B (Timer-C & D)

		clr.b	$fffffa19.w			;Timer-A control (stop)
		clr.b	$fffffa1b.w			;Timer-B control (stop)
		clr.b	$fffffa1d.w			;Timer-C & D control (stop)

		move.w	(sp)+,sr
		rts

deinit:

		move.w	sr,-(sp)
		move.w	#$2700,sr			;Stop interrupts

		move.l	save_hbl,$68.w			;HBL
		move.l	save_vbl,$70.w			;VBL
		move.l	save_timer_a,$134.w		;Timer-A
		move.l	save_timer_b,$120.w		;Timer-B
		move.l	save_timer_c,$114.w		;Timer-C
		move.l	save_timer_d,$110.w		;Timer-D
		move.l	save_acia,$118.w		;ACIA

		lea	save_mfp,a0			;Restore vectors and mfp
		move.b	(a0)+,$fffffa01.w		;// datareg
		move.b	(a0)+,$fffffa03.w		;Active edge
		move.b	(a0)+,$fffffa05.w		;Data direction
		move.b	(a0)+,$fffffa07.w		;Interrupt enable A
		move.b	(a0)+,$fffffa13.w		;Interupt Mask A
		move.b	(a0)+,$fffffa09.w		;Interrupt enable B
		move.b	(a0)+,$fffffa15.w		;Interrupt mask B
		move.b	(a0)+,$fffffa17.w		;Automatic/software end of interupt
		move.b	(a0)+,$fffffa19.w		;Timer A control
		move.b	(a0)+,$fffffa1b.w		;Timer B control
		move.b	(a0)+,$fffffa1d.w		;Timer C & D control
		move.b	(a0)+,$fffffa27.w		;Sync character
		move.b	(a0)+,$fffffa29.w		;USART control
		move.b	(a0)+,$fffffa2b.w		;Receiver status
		move.b	(a0)+,$fffffa2d.w		;Transmitter status
		move.b	(a0)+,$fffffa2f.w		;USART data

		move.w	(sp)+,sr			;Start interrupts


		move.b	save_refresh,$ffff820a.w	;Restore refreshrate
		move.b	save_res,$ffff8260.w		;Restore resolution
		move.b	#$8,$fffffc02.w			;Enable mouse

		lea	save_screenadr,a0		;Restore screenaddress
		move.b	(a0)+,$ffff8201.w		;
		move.b	(a0)+,$ffff8203.w		;
		move.b	(a0)+,$ffff820d.w		;

		movem.l	save_pal,d0-d7			;Restore palette
		movem.l	d0-d7,$ffff8240.w		;

		rts


timer_a:
timer_c:
timer_d:
acia:
hbl:		rte


		section	data

vbl_count:	dc.w	0

gfx:		incbin	'data\gfx.4pl'
		even

sprite:		incbin	'data\sprite.4pl'
		even

rasters:	incbin	'data\rasters.12b'
		even


tb_counter:	dc.w	198

raster_addr:	dc.l	rasters



		section	bss

scr1:		ds.l	1
scr2:		ds.l	1
screen1:	ds.b	160*247+256
screen2:	ds.b	160*247+256

save_hbl:	ds.l	1				;HBL vector
save_vbl:	ds.l	1				;VBL vector
save_timer_a:	ds.l	1				;Timer-A vector
save_timer_b:	ds.l	1				;Timer-B vector
save_timer_c:	ds.l	1				;Timer-C vector
save_timer_d:	ds.l	1				;Timer-D vector
save_acia:	ds.l	1				;ACIA vector
save_mfp:	ds.b	16				;MFP
save_res:	ds.w	1				;Resolution
save_refresh:	ds.w	1				;Refreshrate
save_screenadr:	ds.l	1				;Screen address
save_stack:	ds.l	1				;User stack
save_pal:	ds.w	16				;Palette

